#! /bin/sh

srcdir=`dirname $0`
test -z "$srcdir" && srcdir=.

PKG_NAME="WifiScanner"

(test -f "$srcdir/configure.in" \
  && test -d "$srcdir/src") || {
    echo -n "**Error**: Directory "\`$srcdir\'" does not look like the"
    echo " top-level $PKG_NAME directory"
    exit 1
}

DIE=true
PROJECT="wifiscanner"

ACLOCAL=aclocal
AUTOHEADER=autoheader
AUTOMAKE=automake
AUTOCONF=autoconf

ACVER=`$AUTOCONF --version | grep '^autoconf' | sed 's/.*) *//'`
case "$ACVER" in
'' | 0.* | 1.* | 2.[0-4]* | \
2.5[0-1] | 2.5[0-1][a-z]* )
  cat >&2 <<_EOF_

        You must have autoconf 2.52 or later installed to compile $PROJECT.
        Download the appropriate package for your distribution/OS,
        or get the source tarball at ftp://ftp.gnu.org/pub/gnu/autoconf/
_EOF_
  DIE="exit 1"
  ;;
esac


AMVER=`$AUTOMAKE --version | grep '^automake' | sed 's/.*) *//'`
case "$AMVER" in
'' | 0.* | 1.[0-5]* )
  
  cat >&2 <<_EOF_

        You must have automake 1.6 or later installed to compile $PROJECT.
        Download the appropriate package for your distribution/OS,
        or get the source tarball at ftp://ftp.gnu.org/pub/gnu/automake/
_EOF_
  DIE="exit 1"
  ;;
esac

#
# Apple's Developer Tools have a "libtool" that has nothing to do with
# the GNU libtool; they call the latter "glibtool".  They also call
# libtoolize "glibtoolize".
#
# Check for "glibtool" first.
#
LTVER=`glibtool --version 2>/dev/null | grep ' libtool)' | \
    sed 's/.*libtool) \([0-9][0-9.]*\)[^ ]* .*/\1/'`
if test -z "$LTVER"
then
        LTVER=`libtool --version | grep ' libtool)' | \
            sed 's/.*) \([0-9][0-9.]*\)[^ ]* .*/\1/' `
        LIBTOOLIZE=libtoolize
else
        LIBTOOLIZE=glibtoolize
fi
case "$LTVER" in
'' | 0.* | 1.[0-3]* )

  cat >&2 <<_EOF_

        You must have libtool 1.4 or later installed to compile $PROJECT.
        Download the appropriate package for your distribution/OS,
        or get the source tarball at ftp://ftp.gnu.org/pub/gnu/libtool/
_EOF_
  DIE="exit 1"
  ;;
esac

$DIE

#for dir in . wiretap libpcap-0.9.1-kis ;  do

cd libpcap-0.9.1-kis
mkdir -p bpf/net 2>&1 >/dev/null
cp net/bpf_filter.c bpf/net/bpf_filter.c 2>&1 >/dev/null
make distclean
./configure
make all
cd ..
touch wiretap/ChangeLog

for dir in . wiretap ;  do
  echo processing $dir  
  (
    cd $dir
    echo $ACLOCAL $ACLOCAL_FLAGS
    $ACLOCAL $ACLOCAL_FLAGS || exit 1
    if [ "$dir" = "wiretap" ] ; then
        #
        # We do NOT want libtoolize overwriting our versions of config.guess and
        # config.sub, so move them away and then move them back.
        # We don't omit "--force", as we want libtoolize to install other files
        # without whining.
        #
        LTARGS=" --copy --force"
        echo $LIBTOOLIZE $LTARGS
        $LIBTOOLIZE $LTARGS || exit 1
        rm -f config.guess config.sub
    fi
    echo $AUTOHEADER
    $AUTOHEADER || exit 1
    echo $LIBTOOLIZE
    $LIBTOOLIZE --copy --force || exit 1
    echo $AUTOMAKE --add-missing --gnu $am_opt
    $AUTOMAKE --add-missing --gnu --copy $am_opt || exit 1
    echo $AUTOCONF
    $AUTOCONF || exit 1
  ) || exit 1
done

#./configure "$@" || exit 1

rm -f config.cache wiretap/config.cache
echo


if test -z "$AUTOGEN_SUBDIR_MODE"; then

    echo "Running configure."
    ./configure --enable-maintainer-mode"$@"
    echo
    echo 'run "make"'
    echo
else
    echo
    echo "Now type \"./configure [options]\" and \"make\" to compile $PROJECT."
    echo
fi

