#!/usr/bin/perl -w

# Author : Alain Thivillon (HSC)
# Licence : BSD
# Version : ? alpha   :)

use Date::Parse;
use GD;

# Format des donnes Wifiscanner :
# 03/19/2004 14:42:44.623,"tsunami",10(+0),---,AP,002,000,FF:FF:FF:FF:FF:FF,00:40:96:33:33:33,00:40:96:33:33:33,1Mb/s,AP Base (dedicated),Radio only,BEACON
#

## WARNING : The generated graph is only for information, it's now purely exact
## ATTENTION : Ce script est seulement pour information, les graphiques gnrs ne sont pas purement exacts.

# lire les donnes GPS venant de l'enregistrement d'une track par GPSDrive

if ($#ARGV<1) {
  print STDERR "merge-data <data-gps> <data-wifi>\n";
  exit 1;
}

my $gps = $ARGV[0];
my $wifi = $ARGV[1];
my %coord = ();

my $minlong=200;
my $maxlong=-200;
my $minlat=90;
my $maxlat=-90;


open GPS,"<$gps" or die "can not open gps : $!";
while (defined(my $line=<GPS>)) {
  next if $line =~ /1001/;
  #my ($long,$lat,$alt,$wday,$month,$day,$hour) = ($line =~ /\s+([\d\.]+)\s+([\d\.]+)\s+([\d\.]+)\s+(\S+)\s+(\S+)\s+(\d+)\s+(\S+)/);
  chomp $line,
  my ($long,$lat,$alt,$rest) = ($line =~ /\s+([\d\.]+)\s+([\d\.]+)\s+([\d\.]+)\s+(.*)$/);
  my $tm = str2time($rest);
  #print $tm,"\n";
  $coord{$tm}{'lat'} = $long;
  $coord{$tm}{'long'} = $lat;
  $coord{$tm}{'halt'} = $alt;
  $minlong=$long if ($long < $minlong);
  $minlat=$lat if ($lat < $minlat);
  $maxlong=$long if ($long > $maxlong);
  $maxlat=$lat if ($lat > $maxlat);
}
close GPS;

my %wifipoints=();

my $indexcol=0;
open WIFI,"<$wifi" or die "can not open wifi : $!";
while (defined(my $line=<WIFI>)) {
  next unless $line =~ /AP Base/;
  next if $line =~ /\"test\"/;
  chomp $line;
  my ($date,$dumb,$power,$dumb2,$dumb3,$dumb4,$ap) = 
     ($line =~ /^([^,]+),(.*,AP),(\d+),([^,]+),([^,]+),([^,]+),([^,]+)/);
  #next if $ap =~ /00:40:96:FF:FF:FF/i;         # suppression de l'AP 00:40:96:FF:FF:FF
  next if $ap =~ //i;
  #next unless $ap =~ /00:40:96:FF:FF:FF/i;     # suppression de toutes les AP sauf 00:40:96:FF:FF:FF
  if (!exists($aps{$ap})) {
    print STDERR "AP :", $ap, " - Couleur ", $indexcol, "\n";
    $aps{$ap} = $indexcol;
    $indexcol = ($indexcol+1)%15;
  }
  my $tm= str2time($date);
  #print $date," ",$tm,"\n";
  if (exists($coord{$tm})) {
    my $lat = $coord{$tm}{'lat'};
    my $long = $coord{$tm}{'long'};
    if (exists($wifipoints{$lat}{$long})) {
       if ($power > $wifipoints{$lat}{$long}) {
         $wifipoints{$lat}{$long}=$power;
         $appoint{$lat}{$long}=$ap;
       }
    } 
    else {
      $wifipoints{$lat}{$long}=$power;
      $appoint{$lat}{$long}=$ap;
    }
  }
}
close WIFI;

my $im=new GD::Image(1024,768);
my $white = $im->colorAllocate(255,255,255);
my @colors=();

$colors[0] =  $im->colorAllocate(255,0,0);
$colors[1] =  $im->colorAllocate(0,128,40);
$colors[2] =  $im->colorAllocate(0,0,255);
$colors[3] =  $im->colorAllocate(0,0,0);
$colors[4] =  $im->colorAllocate(255,255,0);
$colors[5] =  $im->colorAllocate(255,0,255);
$colors[4] =  $im->colorAllocate(0,255,255);
$colors[6] =  $im->colorAllocate(255,128,128);
$colors[7] =  $im->colorAllocate(128,255,196);
$colors[8] =  $im->colorAllocate(128,128,255);
$colors[9] =  $im->colorAllocate(000,128,255);
$colors[10] = $im->colorAllocate(128,000,255);
$colors[11] = $im->colorAllocate(255,128,000);
$colors[12] = $im->colorAllocate(255,128,064);
$colors[13] = $im->colorAllocate(255,064,128);
$colors[14] = $im->colorAllocate(064,255,128);

my $colordep = $im->colorAllocate(0,255,0);
#$im->transparent($white);
my $scalex = 1024.0 / ($maxlat - $minlat);
my $scaley = 768.0 / ($maxlong - $minlong);
my $scale = $scalex;
$scale = $scaley if ($scaley < $scalex) ;
$scale = 0.95 * $scale;

print STDERR "$scale $minlong $minlat $maxlong $maxlat\n";
foreach $key (keys(%coord)) {
  $o = $coord{$key}{'long'};
  $p = $coord{$key}{'lat'};
  $im->setPixel(($o-$minlat)*$scale+5,760-($p-$minlong)*$scale,$colordep);
  #printf STDERR "%s %s %f %f\n", $o, $p , ($p-$minlong)*$scale, ($o-$minlat)*$scale;
}

foreach $p (keys(%wifipoints)) {
  foreach $o (keys(%{$wifipoints{$p}})) {
    #printf "%s\t%s\t%s\n",$o, $p, $wifipoints{$p}{$o};
    my $power = $wifipoints{$p}{$o}/2;
    my $colindex = $colors[$aps{$appoint{$p}{$o}}];
    $im->setPixel(($o-$minlat)*$scale+5,760-($p-$minlong)*$scale,$colindex);
    $im->filledEllipse(($o-$minlat)*$scale+5,760-($p-$minlong)*$scale,$power,$power,$colindex);
    # printf "%s %s %f %f\n", $o, $p , ($p-$minlong)*$scale, ($o-$minlat)*$scale;
  }
}

my $legx=800;
my $legy=10;
foreach $apcol (keys(%aps)) {
  $im->string(gdSmallFont,$legx,$legy,$apcol,$colors[$aps{$apcol}]);
  $legy+=15;
}

binmode STDOUT;
print $im->png;

sub max {
 my $a = shift;
 my $b = shift;
  return $a if ($a > $b);
  return $b;
};

sub min {
 my $a = shift;
 my $b = shift;
  return $a if ($a < $b);
  return $b;
};

