/* Linux Prism II Stumbler - Utility Scan for 802_11 networks under Linux
 * 
 * File : functions.h
 * Project : WifiScanner (c) 2002 Herv Schauer Consultants
 * Usage : This utility is written for use with IEEE 802.11 adapters based
 * on Intersil's PRISM II chipset (PCMCIA).
 * 
 * Base code was from prismstumbler Jan Fernquist <Jan.B.Fernquist@telia.com>
 * and wlanctl from www.linux-wlan.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: functions.h 144 2006-01-03 13:52:09Z poggij $
 */

#ifndef __functions_h__
#define __functions_h__

#include <wlan/p80211types.h>
#include <wlan/p80211msg.h>
#include <wlan/p80211meta.h>
#include <wlan/p80211metamsg.h>
#include <wlan/p80211metastruct.h>
#include <wlan/p80211mgmt.h>

#define CHANNEL_MIN 1
#define CHANNEL_MAX 14

#define HOP_MIN 1
#define HOP_MAX 13

#define IFACE_WLAN_MIN 0
#define IFACE_WLAN_MAX 4

#ifdef RADIOTAP_H_EXIST
/*
 *  * Useful combinations of channel characteristics.
 *   */
#define IEEE80211_CHAN_FHSS \
          (IEEE80211_CHAN_2GHZ | IEEE80211_CHAN_GFSK)
#define IEEE80211_CHAN_A \
          (IEEE80211_CHAN_5GHZ | IEEE80211_CHAN_OFDM)
#define IEEE80211_CHAN_B \
          (IEEE80211_CHAN_2GHZ | IEEE80211_CHAN_CCK)
#define IEEE80211_CHAN_PUREG \
          (IEEE80211_CHAN_2GHZ | IEEE80211_CHAN_OFDM)
#define IEEE80211_CHAN_G \
          (IEEE80211_CHAN_2GHZ | IEEE80211_CHAN_DYN)
#define IEEE80211_CHAN_T \
          (IEEE80211_CHAN_5GHZ | IEEE80211_CHAN_OFDM | IEEE80211_CHAN_TURBO)

#define IEEE80211_IS_CHAN_FHSS(_flags) \
          ((_flags & IEEE80211_CHAN_FHSS) == IEEE80211_CHAN_FHSS)
#define IEEE80211_IS_CHAN_A(_flags) \
          ((_flags & IEEE80211_CHAN_A) == IEEE80211_CHAN_A)
#define IEEE80211_IS_CHAN_B(_flags) \
          ((_flags & IEEE80211_CHAN_B) == IEEE80211_CHAN_B)
#define IEEE80211_IS_CHAN_PUREG(_flags) \
          ((_flags & IEEE80211_CHAN_PUREG) == IEEE80211_CHAN_PUREG)
#define IEEE80211_IS_CHAN_G(_flags) \
          ((_flags & IEEE80211_CHAN_G) == IEEE80211_CHAN_G)
#define IEEE80211_IS_CHAN_T(_flags) \
          ((_flags & IEEE80211_CHAN_T) == IEEE80211_CHAN_T)

#define IEEE80211_RADIOTAP_TSFT 0
#define IEEE80211_RADIOTAP_FLAGS 1
#define IEEE80211_RADIOTAP_RATE 2
#define IEEE80211_RADIOTAP_CHANNEL 3
#define IEEE80211_RADIOTAP_FHSS 4
#define IEEE80211_RADIOTAP_DBM_ANTSIGNAL 5
#define IEEE80211_RADIOTAP_DBM_ANTNOISE 6
#define IEEE80211_RADIOTAP_LOCK_QUALITY 7
#define IEEE80211_RADIOTAP_TX_ATTENUATION 8
#define IEEE80211_RADIOTAP_DB_TX_ATTENUATION 9
#define IEEE80211_RADIOTAP_DBM_TX_POWER 10
#define IEEE80211_RADIOTAP_ANTENNA 11
#define IEEE80211_RADIOTAP_DB_ANTSIGNAL 12
#define IEEE80211_RADIOTAP_DB_ANTNOISE 13
#define IEEE80211_RADIOTAP_EXT 31

#endif

int processPacket(UINT8 * packet, int len);
void ParseCommandLine(int argc, char **argv, ConfigStruct * config);
void ChannelToStr(char *strChannel, UINT8 Channel, UINT8 SChannel);
char *safe_strncpy(char *dst, const char *src, int size);
int safe_snprintf(char *s, int size, char *fmt, ...);
void *memcpy_buff(void *dest, const void *src, size_t n);
void InitConfiguration(ConfigStruct * config);
void FillRadioData(p80211_caphdr_t * wlan_header, int DataLink,
                   UINT8 * Packet);
int CalculateOffset(int DataLink);
#endif
