/* Linux Prism II Stumbler - Utility Scan for 802_11 networks under Linux
 * 
 * File : $Name$
 * Project : WifiScanner (c) 2002 Herv Schauer Consultants
 * Usage : This utility is written for use with IEEE 802.11 adapters based
 * on Intersil's PRISM II chipset (PCMCIA).
 * 
 * Base code was from prismstumbler Jan Fernquist <Jan.B.Fernquist@telia.com>
 * and wlanctl from www.linux-wlan.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: hostap.c 144 2006-01-03 13:52:09Z poggij $
 */

// A lot of think is get from kismet
//  http://www.kismetwireless.net/

#include <include.h>
#include <src/hostap.h>
#include <src/crt_io.h>
#include <src/functions.h>

static char *ID = "$Id: hostap.c 144 2006-01-03 13:52:09Z poggij $";

// All extern value you want
extern UINT8 SingleChannel;
extern UINT8 TypeOfCard;
extern p80211_caphdr_t wlan_header;

static CaptureArg ca;
static char errbuf[PCAP_ERRBUF_SIZE];
static UINT8 wlan_payload[MAX_BUFFER_SIZE];

//-------------
int selectChannelHOSTAP(char *devname, int channel)
{
  char str[80];
  int result = 0;

  sprintf(str, "iwconfig %s channel %d 2> /dev/null", devname, channel);
  debug(3, str, "\n");
  result += system(str);

  //return result;
  // No error return, because it's possible to change to a forbidden channel
  //  So we don't wand that the program stop because of this restriction
  return 0;
}

int shutCardHOSTAP(char *devname)
{
  char str[80];
  int result = 0;

  // Turn off monitor mode
  sprintf(str, "iwconfig %s mode managed", devname);
  debug(3, "%s\n", str);
  result += system(str);

  sprintf(str, "iwconfig %s channel 1", devname);
  debug(3, "%s\n", str);
  result += system(str);

  sprintf(str, "ifconfig %s -promisc up", devname);
  debug(3, "%s\n", str);
  result += system(str);

  return result;
}

int openCardHOSTAP(char *devname)
{
  char str[80];
  int result = 0;

  // Turn on monitor mode
  sprintf(str, "prism2_param %s monitor_type 1", devname);
  debug(3, "%s\n", str);
  result += system(str);

  sprintf(str, "iwconfig %s mode monitor_type", devname);
  debug(3, "%s\n", str);
  result += system(str);

  sprintf(str, "iwconfig %s channel 1", devname);
  debug(3, "%s\n", str);
  result += system(str);

  sprintf(str, "ifconfig %s promisc up", devname);
  debug(3, "%s\n", str);
  result += system(str);

  return result;
}


// Get packet from card
int getPacketHOSTAP(UINT8 * buf, int maxlen)
{
  struct pcap_pkthdr pktHdr;
  u_char *RadioPacket;
  fd_set rs;

  FD_ZERO(&rs);
  FD_SET(0, &rs);

  RadioPacket = (u_char *) pcap_next(ca.pcap, &pktHdr);

  if (RadioPacket != NULL) {
    // If no problem and packet is enought big (with data)
    if (pktHdr.len >= sizeof(p80211msg_lnxind_wlansniffrm_t)) {
      if (pktHdr.len > MAX_BUFFER_SIZE) {
        debug(1, "ERROR : Packet is TOOO BIG size=%d\n", pktHdr.len);
        // Sometimes Wlan-NG return an enormous empty packet ...
        //DumpHexPaquets(RealTime_WND, buf, 0x1B0);
        return 0;
      } else {
        FillRadioData(&wlan_header, ca.DataLink, RadioPacket);
        memcpy_buff(buf, (RadioPacket + ca.offset),
                    MIN_OF((pktHdr.len - ca.offset), maxlen));
        return MAX_OF(0,
                      (pktHdr.len -
                       sizeof(p80211msg_lnxind_wlansniffrm_t)));
      }
    }
  }
  return (0);                   /* Noting to read */
}

int openPacketHOSTAP(char *devname)
{
  ca.pcap = pcap_open_live(devname, 3000, 1, 0, errbuf);
  if (ca.pcap) {
    pcap_setnonblock(ca.pcap, 1, errbuf);
    ca.DataLink = pcap_datalink(ca.pcap);
    ca.offset = CalculateOffset(ca.DataLink);
    return 1;
  }
  return -1;
}

void closePacketHOSTAP(void)
{
  pcap_close(ca.pcap);
}
