/* Linux Prism II Stumbler - Utility Scan for 802_11 networks under Linux
 * 
 * File : $Name$
 * Project : WifiScanner (c) 2002 Herv Schauer Consultants
 * Usage : This utility is written for use with IEEE 802.11 adapters based
 * on Intersil's PRISM II chipset (PCMCIA).
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: ipw.c 144 2006-01-03 13:52:09Z poggij $
 */

#include <include.h>
#include <src/ipw.h>
#include <src/wlan-ng.h>
#include <src/functions.h>
#include <src/crt_io.h>

static char *ID = "$Id: ipw.c 144 2006-01-03 13:52:09Z poggij $";

// All extern value you want
//extern unsigned int DebugLevel;
//extern UINT8 SingleChannel;
//extern UINT8 TypeOfCard;
extern ConfigStruct config;
extern p80211_caphdr_t wlan_header;
extern WINDOW *Sum_WND, *RealTime_WND;

static CaptureArg ca;
static char errbuf[PCAP_ERRBUF_SIZE];

//-------------
int selectChannelIPW(char *devname, int channel)
{
#define STR_MAX 80
  char str[STR_MAX];
  int result = 0;

  //snprintf(str, STR_MAX, "iwconfig %s mode monitor channel %d 2>/dev/null",
  //         devname, channel);
  snprintf(str, STR_MAX, "iwpriv %s monitor 1 %d 2>/dev/null",
           devname, channel);
  debug(3, "%s\n", str);
  result += system(str);
  //return 0;
  return result;
}

int shutCardIPW(char *devname)
{
#define STR_MAX 80
  char str[STR_MAX];
  int result = 0;

  snprintf(str, STR_MAX, "iwconfig %s mode managed", devname);
  debug(3, "%s\n", str);
  result += system(str);

  snprintf(str, STR_MAX, "ifconfig %s -promisc up", devname);
  debug(3, "%s\n", str);
  result += system(str);

  return result;
}

int openCardIPW(char *devname)
{
  char str[STR_MAX];
  int result = 0;

  // Turn on monitor mode
  snprintf(str, STR_MAX, "iwconfig %s mode monitor", devname);
  debug(3, "%s\n", str);
  result += system(str);

  snprintf(str, STR_MAX, "ifconfig %s promisc up", devname);
  debug(3, "%s\n", str);
  result += system(str);

  return result;
}

// Get packet from card
int getPacketIPW(UINT8 * buf, int maxlen)
{
  struct pcap_pkthdr pktHdr;
  u_char *RadioPacket;
  fd_set rs;

  FD_ZERO(&rs);
  FD_SET(0, &rs);

  RadioPacket = (u_char *) pcap_next(ca.pcap, &pktHdr);
  if (RadioPacket != NULL) {
#if 0
    // Function to dump in hexa the message send or sent to the driver
    DumpHexPaquets(RealTime_WND, RadioPacket, 0x40);
#endif
    if (ca.DataLink == DLT_IEEE802_11_RADIO) {
      // calculate the offset = RadioPacket[2];
      ca.offset = (UINT16) RadioPacket[2];
      debug(3, "DLT_IEEE802_11_RADIO >> ca.offset = %x\n", ca.offset);
    }
    debug(3, "ca.offset = %x - pktHdr.len = %x\n", ca.offset, pktHdr.len);
    // If no problem and packet is enought big (with data)
    if (pktHdr.len > MAX_BUFFER_SIZE) {
      debug(1, "ERROR : Packet is TOOO BIG size=%d\n", pktHdr.len);
      // Sometimes return an enormous empty packet ...
      //DumpHexPaquets(RealTime_WND, buf, 0x1B0);
      return 0;
    } else {
      memcpy_buff(buf, (RadioPacket + ca.offset),
                  MIN_OF((pktHdr.len - ca.offset), maxlen));
      FillRadioData(&wlan_header, ca.DataLink, RadioPacket);
      return MAX_OF(0, (pktHdr.len - ca.offset));
    }
  }
  return (0);                   /* Noting to read */
}

int openPacketIPW(char *devname)
{
  ca.pcap = pcap_open_live(devname, MAX_PACKET_LEN, 0, 1000, errbuf);

  if (ca.pcap) {
    pcap_setnonblock(ca.pcap, 1, errbuf);
    ca.DataLink = pcap_datalink(ca.pcap);
    ca.offset = CalculateOffset(ca.DataLink);
    return 1;
  }
  return -1;
}

void closePacketIPW(void)
{
  pcap_close(ca.pcap);
}
